/**
 * 
 * RSSGrabber - a tool to grab RSS/RDF feeds and extract
 * the links: The links are then captured using the GNU Wget
 * utility and saved to disk for offline viewing.
 *
 */

import java.net.*;
import java.io.*;
import java.util.*;

public class Grab {

	public static void main(String[] args) {
		new Grab(args);
	}

	public Grab(String[] args) {
		
		printCopyright();
		
		if (args.length == 0) {
			out("Usage: rssgrabber [ feed URL 1] [ feed URL 2] ...");
			System.exit(1);
		}

		// Grab each feed in turn
		for (int i = 0; i < args.length; i++) {
			try {	
				URL u = new URL(args[i]);
				String rss = getURL(u);

				// Extract the URLs from the RSS feed by looking
				// for <item rdf:about="http://[url]" tags
				Vector urls = new Vector();
				int sp = rss.indexOf("rdf:about=\"http://");
				while (sp != -1) {

					int rsp = rss.indexOf("\"", sp) + 1;
					int esp = rss.indexOf("\"", rsp);
					String ur = rss.substring(rsp, esp);
					out("Found URL: " + ur);
					urls.add(ur);

					sp = rss.indexOf("rdf:about=\"http://", sp + 2);
				}

				
				// Create a directory for items we download
				// from the feed from the hostname of the
				// feed (no http:// and upto the first slash)
				/*
				String host = "";
				try {
					host = args[i].substring(7, args[i].indexOf("/", 8));
					Runtime.getRuntime().exec("mkdir " + host).waitFor();
				}
				catch (Exception e) {
					out("Error occurred creating '" + host + "' directory.");
					e.printStackTrace();
					System.exit(1);
				}
				*/
				
				// Loop through each found URL and download
				// the page with WGET
				for (int z = 0; z < urls.size(); z++) {
					try {
						out("Downloading: " + urls.get(z).toString());
						Runtime.getRuntime().exec("wget " + urls.get(z).toString());
					}
					catch (Exception e) {
						out("Error occurred retrieving: " + urls.get(z).toString());
						e.printStackTrace();
						System.exit(1);
					}
				}
				
			}
			catch (Exception e) {
				out("An error occurred resolving: " + args[i]);
				e.printStackTrace();
				System.exit(1);
			}

		}
		
	}

	/**
	 * Outputs the copyright message
	 */
	public void printCopyright() {
		out("RSSGrabber 1.0, by R.Rawson-Tetley.");
		out("Downloads RDF/RSS feeds and caches the links as local files");
		out("for offline viewing.");
		out("");
		out("This software is provided with no warranty.");
		out("");
	}

	/**
	 * Outputs to to wherever output should go
	 */
	public void out(String m) {
		System.out.println(m);
	}

	/**
	 * Returns a URL's contents as a String
	 */
	public String getURL(URL u) {
		try {
			BufferedReader in = new BufferedReader(new InputStreamReader(u.openStream()));
			String inputLine;
			StringBuffer buf = new StringBuffer();
		
			while ((inputLine = in.readLine()) != null) {
				buf.append(inputLine);
			}
			in.close();
			return buf.toString();
		}
		catch (Exception e) {
			out("An error occurred reading from: " + u.toString());
			e.printStackTrace();
			System.exit(1);
		}
		return null;
	}

}
